package com.ahazou.photo_editor_sdk.instagram_ui;

import android.content.Context;
import android.graphics.drawable.GradientDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.ahazou.photo_editor_sdk.R;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import ly.img.android.pesdk.ui.panels.item.FontItem;

public class InstagramFontAdapter extends RecyclerView.Adapter<InstagramFontAdapter.FontViewHolder> {

    final private ListItemClickListener onClickListener;

    private List<FontItem> fontData;

    private int selectedPosition = -1;

    public interface ListItemClickListener {
        void onColorListItemClick(FontItem clickedItem);
    }

    public InstagramFontAdapter(ListItemClickListener listener) {
        onClickListener = listener;
    }

    @NotNull
    @Override
    public InstagramFontAdapter.FontViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        Context context = parent.getContext();
        int layoutIdForListItem = R.layout.iui_color_item;
        LayoutInflater inflater = LayoutInflater.from(context);

        View view = inflater.inflate(layoutIdForListItem, parent, false);
        InstagramFontAdapter.FontViewHolder viewHolder = new InstagramFontAdapter.FontViewHolder(view);

        return viewHolder;
    }

    @Override
    public void onBindViewHolder(InstagramFontAdapter.FontViewHolder holder, int position) {
        holder.bind(position);
    }

    @Override
    public void onBindViewHolder(@NonNull InstagramFontAdapter.FontViewHolder holder,
                                 int position, @Nullable List<Object> payloads) {
        if (payloads != null && payloads.size() > 0) {
            holder.setSelectionState(selectedPosition == position);
        } else {
            onBindViewHolder(holder, position);
        }
    }

    @Override
    public int getItemCount() {
        return fontData.size();
    }

    protected class FontViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        private FontItem lastBoundData;
        private View listItemFontView;

        public FontViewHolder (View itemView) {
            super(itemView);
            listItemFontView = itemView.findViewById(R.id.iui_color_item);
            itemView.setOnClickListener(this);
        }

        protected void bind(int listIndex) {
            lastBoundData = fontData.get(listIndex);
            GradientDrawable gradientDrawable = (GradientDrawable)
                    itemView.getResources().getDrawable(R.drawable.iui_rounded_color_list_item_background);

            //gradientDrawable.setColor(lastBoundData.getData().getColor());

            listItemFontView.setBackgroundDrawable(gradientDrawable);
            setSelectionState(selectedPosition == listIndex);
        }

        @Override
        public void onClick(View v) {
            onClickListener.onColorListItemClick(lastBoundData);
        }

        public void setSelectionState(boolean isSelected) {
            itemView.setSelected(isSelected);
        }
    }
}
