package com.ahazou.photo_editor_sdk.instagram_ui;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.recyclerview.widget.RecyclerView;

import com.ahazou.photo_editor_sdk.R;
import com.ahazou.photo_editor_sdk.utils.CustomImageStickerAsset;

import java.util.ArrayList;
import java.util.List;

import ly.img.android.pesdk.backend.model.config.ImageStickerAsset;
import ly.img.android.pesdk.ui.widgets.ImageSourceView;


/**
 * Created by niklasbachmann on 28.11.17.
 */

public class InstagramStickerAdapter extends RecyclerView.Adapter<InstagramStickerAdapter.StickerViewHolder> {

    final private ListItemClickListener mOnClickListener;

    private List<CustomImageStickerAsset> stickerData = new ArrayList<>();

    public InstagramStickerAdapter(ListItemClickListener listener) {
        mOnClickListener = listener;

    }

    public void setStickerData(List<CustomImageStickerAsset> stickerData) {
        this.stickerData = stickerData;

    }

    @Override
    public StickerViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        Context context = parent.getContext();
        int layoutIdForListItem = R.layout.iui_sticker_item;
        LayoutInflater inflater = LayoutInflater.from(context);
        View view = inflater.inflate(layoutIdForListItem, parent, false);
        return new StickerViewHolder(view);
    }

    @Override
    public void onBindViewHolder(StickerViewHolder holder, int position) {
        holder.bind(position);
    }

    @Override
    public int getItemCount() {
        return stickerData.size();
    }

    public interface ListItemClickListener {
        void onStickerListItemClick(ImageStickerAsset clickedSticker);
    }

    public List<CustomImageStickerAsset> getStickerData() {
        return stickerData;
    }

    class StickerViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        ImageSourceView listItemStickerView;
        ImageStickerAsset lastBindedSticker;


        public StickerViewHolder(View itemView) {
            super(itemView);
            listItemStickerView = itemView.findViewById(R.id.iui_sticker_item);
            itemView.setOnClickListener(this);
        }

        void bind(int listIndex) {
            lastBindedSticker = stickerData.get(listIndex).getImage();
            listItemStickerView.setImageSource(lastBindedSticker.getStickerSource());
        }

        @Override
        public void onClick(View v) {
            mOnClickListener.onStickerListItemClick(lastBindedSticker);
        }
    }
}
