package com.ahazou.photo_editor_sdk.instagram_ui.panel;

import android.content.Context;
import android.content.Intent;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.ahazou.photo_editor_sdk.R;
import com.ahazou.photo_editor_sdk.instagram_ui.InstagramStickerAdapter;
import com.ahazou.photo_editor_sdk.instagram_ui.InstagramUIActivity;
import com.ahazou.photo_editor_sdk.utils.CustomImageStickerAsset;

import java.util.List;

import ly.img.android.pesdk.backend.model.config.ImageStickerAsset;
import ly.img.android.pesdk.backend.model.state.EditorShowState;
import ly.img.android.pesdk.backend.model.state.LayerListSettings;
import ly.img.android.pesdk.backend.model.state.layer.ImageStickerLayerSettings;
import ly.img.android.pesdk.backend.model.state.layer.SpriteLayerSettings;
import ly.img.android.pesdk.backend.model.state.manager.StateHandler;
import ly.img.android.pesdk.backend.views.abstracts.ImgLyUIRelativeContainer;

/**
 * Created by niklasbachmann on 06.12.17.
 */
public class LogoPanel extends ImgLyUIRelativeContainer implements
        InstagramStickerAdapter.ListItemClickListener {

    private View header;
    private RecyclerView stickerList;
    private EditorShowState editorState;
    private LayerListSettings layerListSettings;
    private OnPanelCloseListener onPanelCloseListener = null;
    private InstagramUIActivity activity;
    private InstagramStickerAdapter stickerAdapter;


    public LogoPanel(Context context) {
        this(context, null);
    }

    public LogoPanel(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public LogoPanel(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        LayoutInflater inflater = LayoutInflater.from(context);
        inflater.inflate(R.layout.instagram_ui_sticker, this);
        init();
    }

    private void init() {
        setVisibility(GONE);
        stickerList = findViewById(R.id.rv_stickers);
        header = findViewById(R.id.header);
        header.setOnClickListener(v -> onPanelClose());
        stickerAdapter = new InstagramStickerAdapter(this);
        stickerAdapter.setStickerData(getStickerData());
        stickerList.setLayoutManager(new GridLayoutManager(getContext(), 5));
        stickerList.setHasFixedSize(true);
        stickerList.setAdapter(stickerAdapter);
    }

    public List<CustomImageStickerAsset> getStickerData() {
        return stickerAdapter.getStickerData();
    }

    public InstagramStickerAdapter getStickerAdapter() {
        return stickerAdapter;
    }

    @Override
    protected void onAttachedToUI(StateHandler stateHandler) {
        super.onAttachedToUI(stateHandler);
        editorState = stateHandler.getStateModel(EditorShowState.class);
    }

    public void switchVisibility(boolean isVisible) {
        if (isVisible) {
            setVisibility(VISIBLE);
            getLayerListSettings().setSelected(null);
        } else setVisibility(GONE);
    }

    @Override
    public void onStickerListItemClick(ImageStickerAsset clickedSticker) {
        if( "imgly_ic_add_new_logo".equals(clickedSticker.getId()) ) {
            Intent intent = new Intent();
            intent.setType("image/*");
            intent.setAction(Intent.ACTION_GET_CONTENT);
            activity.startActivityForResult(
                    Intent.createChooser(intent, "Select Picture"),
                    InstagramUIActivity.SELECT_IMAGE);
            return;
        }
        addImageSticker(clickedSticker);
        onPanelClose();
    }

    private void addImageSticker(ImageStickerAsset config) {
        SpriteLayerSettings spriteLayerSettings = new ImageStickerLayerSettings(config);
        getLayerListSettings().addLayer(spriteLayerSettings);
        getLayerListSettings().setSelected(spriteLayerSettings);
    }

    protected LayerListSettings getLayerListSettings() {
        if (this.layerListSettings == null) {
            StateHandler stateHandler = getStateHandler();
            if (stateHandler == null) return null;
            this.layerListSettings = stateHandler.getStateModel(LayerListSettings.class);
        }
        return this.layerListSettings;
    }

    public void setOnPanelCloseListener(OnPanelCloseListener onPanelCloseListener) {
        this.onPanelCloseListener = onPanelCloseListener;
    }

    public void onPanelClose() {
        if (onPanelCloseListener != null) {
            onPanelCloseListener.onPanelClose(this);
        }
    }

    public void setActivity(InstagramUIActivity activity) {
        this.activity = activity;
    }
}
